package eu.dnetlib.repo.manager.controllers;

import eu.dnetlib.api.functionality.ValidatorServiceException;
import eu.dnetlib.domain.functionality.validator.StoredJob;
import eu.dnetlib.repo.manager.service.ValidatorServiceImpl;
import eu.dnetlib.repo.manager.shared.InterfaceInformation;
import eu.dnetlib.repo.manager.shared.ValidationServiceException;
import eu.dnetlib.domain.functionality.validator.JobForValidation;
import eu.dnetlib.domain.functionality.validator.RuleSet;

import java.util.*;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiParam;
import org.json.JSONException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;


@RestController
@RequestMapping(value = "/validator")
@Api(description = "Validator API",  tags = {"validator"})
public class ValidatorController {

    @Autowired
    ValidatorServiceImpl validatorService;

    @RequestMapping(value = "/submitJobForValidation",method = RequestMethod.POST,
            consumes = MediaType.APPLICATION_JSON_VALUE,
            produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    @PreAuthorize("hasRole('ROLE_USER') and #jobForValidation.userEmail == authentication.userInfo.email")
    public JobForValidation submitJobForValidation(@RequestBody JobForValidation jobForValidation) throws ValidatorServiceException {
        return validatorService.submitJobForValidation(jobForValidation);
    }

    @RequestMapping(value = "/reSubmitJobForValidation/{email}/{jobId}",method = RequestMethod.POST,
            consumes = MediaType.APPLICATION_JSON_VALUE,
            produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    @PreAuthorize("hasRole('ROLE_USER') and #email == authentication.userInfo.email")
    public ResponseEntity<Object> reSubmitJobForValidation(@PathVariable("email") String email,
                                                           @PathVariable("jobId") String jobId) throws JSONException, ValidatorServiceException {
        return validatorService.reSubmitJobForValidation(email, jobId);
    }

    @RequestMapping(value = "/getRuleSets/{mode}" , method = RequestMethod.GET,produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    public List<RuleSet> getRuleSets(@PathVariable("mode") String mode) {
        return validatorService.getRuleSets(mode);
    }

    @RequestMapping(value = "/getSetsOfRepository" , method = RequestMethod.GET,produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    public List<String> getSetsOfRepository(@RequestParam(value = "url", required = true) String url) {
        return validatorService.getSetsOfRepository(url);
    }

    @RequestMapping(value = "/identifyRepository" , method = RequestMethod.GET,produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    public boolean identifyRepo(@RequestParam(value = "url", required = true) String url) {
        return validatorService.identifyRepo(url);
    }

    @RequestMapping(value = "/getRuleSet/{acronym}" , method = RequestMethod.GET,produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    public RuleSet getRuleSet(@PathVariable("acronym") String acronym) {
        return validatorService.getRuleSet(acronym);
    }

    @RequestMapping(value = "/getStoredJobsNew" , method = RequestMethod.GET,produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    @PreAuthorize("hasRole('ROLE_USER')")
    public List<StoredJob> getStoredJobsNew(@RequestParam("user") @ApiParam(value = "User email", required = true) String user,
                                            @RequestParam(value = "jobType", required = false)
                                            @ApiParam(value = "Equals to filter job type on validation history page") String jobType,
                                            @RequestParam("offset") @ApiParam(value = "Page number", required = true) String offset,
                                            @RequestParam(value = "limit", required = false,defaultValue = "10") @ApiParam(value = "Null value") String limit,
                                            @RequestParam(value = "dateFrom", required = false) @ApiParam(value = "Null value") String dateFrom,
                                            @RequestParam(value = "dateTo", required = false) @ApiParam(value = "Null value") String dateTo,
                                            @RequestParam("validationStatus") @ApiParam(value = "Equals to filter validation jobs", required = true) String validationStatus
                                            ) throws ValidatorServiceException {
        return validatorService.getStoredJobsNew(user, jobType, offset, limit, dateFrom, dateTo, validationStatus);
    }

    @RequestMapping(value = "/getStoredJobsTotalNumberNew" , method = RequestMethod.GET,produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    public int getStoredJobsTotalNumberNew(String user, String jobType, String validationStatus) throws  ValidatorServiceException {
        return validatorService.getStoredJobsTotalNumberNew(user, jobType, validationStatus);
    }

    @RequestMapping(value = "/getInterfaceInformation" , method = RequestMethod.GET,produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    public InterfaceInformation getInterfaceInformation(@RequestParam(value = "baseUrl", required = true) String baseUrl) throws ValidationServiceException {
        return validatorService.getInterfaceInformation(baseUrl);
    }


}
